using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

namespace DigiDesign
{
    public class LineDrawing
    {
        public LineType type;
        public int locationX, locationY;
        public int startX, startY;
        public int endX, endY;
        public int arcStart, arcSweep;

        //creates a line drawing of type line
        public LineDrawing(int _startx, int _starty, int _endx, int _endy) {
            startX = _startx;
            startY = _starty;
            endX = _endx;
            endY = _endy;
            type = LineType.Line;
        }

        //creates a line drawing of type arc
        public LineDrawing(int _startx, int _starty, int _endx, int _endy, int _arcstart, int _arcend) {
            startX = _startx;
            startY = _starty;
            endX = _endx;
            endY = _endy;
            arcStart = _arcstart;
            arcSweep = _arcend;
            type = LineType.Arc;
        }

        //sets location for easy calls
        public void setLocation(int locx, int locy) {
            locationX = locx;
            locationY = locy;
        }

        //draws the drawing at a particular position
        public void drawLineDrawing(Graphics g, Pen p, int locx, int locy) {
            switch (type) {
                case LineType.Arc:
                    g.DrawArc(p, locx + startX, locy + startY,
                        endX - startX, endY - startY,
                        arcStart, arcSweep);
                    break;
                case LineType.Line:
                    g.DrawLine(p, locx + startX, locy + startY,
                        locx + endX, locy + endY);
                    break;
                default:
                    break;
            }
        }

        //draws the drawing at the default position
        public void drawLineDrawing(Graphics g, Pen p) {
            switch (type) {
                case LineType.Arc:
                    g.DrawArc(p, locationX + startX, locationY + startY,
                        locationX + endX, locationY + endY,
                        arcStart, arcSweep);
                    break;
                case LineType.Line:
                    g.DrawLine(p, locationX + startX, locationY + startY,
                        locationX + endX, locationY + endY);
                    break;
                default:
                    break;
            }
        }
    }

    public enum LineType{
        Arc,
        Line
    }
}
