using System;
using System.Collections.Generic;
using System.Text;
using Point = System.Drawing.Point;
using Rectangle = System.Drawing.Rectangle;

namespace DigiDesign
{
    public class AndGate : DigitalObject
    {

        public AndGate() {
            numInputs = 2;
            numOutputs = 1;

            inputs = new DigitalObject[numInputs];
            outputs = new DigitalObject[numOutputs];

            toolType = ToolboxStrip.DigiTool.AndGate;

            inputPoints = new Point[numInputs];
            outputPoints = new Point[numOutputs];

            populatePoints();
            populateLineDrawing();
        }

        private void populateLineDrawing() {
            lineDrawing = new LineDrawing[7];
            lineDrawing[0] = new LineDrawing(24, 0, 36, 24, 270, 180);//arc
            lineDrawing[1] = new LineDrawing(5, 0, 30, 0);//top line
            lineDrawing[2] = new LineDrawing(5, 0, 5, 24);//left
            lineDrawing[3] = new LineDrawing(5, 24, 30, 24);//bottom
            lineDrawing[4] = new LineDrawing(0, 6, 5, 6);//input 1
            lineDrawing[5] = new LineDrawing(0, 18, 5, 18);//input 2
            lineDrawing[6] = new LineDrawing(36, 12, 42, 12);//output
        }

        private void populatePoints() {
            inputPoints[0] = new Point(0, 6);
            inputPoints[1] = new Point(0, 18);
            outputPoints[0] = new Point(42, 12);
        }

    }

    public class OrGate : DigitalObject
    {

        public OrGate() {
            numInputs = 2;
            numOutputs = 1;

            inputs = new DigitalObject[numInputs];
            outputs = new DigitalObject[numOutputs];

            toolType = ToolboxStrip.DigiTool.OrGate;

            inputPoints = new Point[numInputs];
            outputPoints = new Point[numOutputs];

            populatePoints();
            populateLineDrawing();
        }

        private void populateLineDrawing() {
            lineDrawing = new LineDrawing[7];
            lineDrawing[0] = new LineDrawing(5, 0, 42, 24, 270, 180);//arc
            lineDrawing[1] = new LineDrawing(8, 0, 22, 0);//top line
            lineDrawing[2] = new LineDrawing(5, 0, 16, 24, 270, 180);//left
            lineDrawing[3] = new LineDrawing(8, 24, 22, 24);//bottom
            lineDrawing[4] = new LineDrawing(4, 6, 14, 6);//input 1
            lineDrawing[5] = new LineDrawing(4, 18, 14, 18);//input 2
            lineDrawing[6] = new LineDrawing(42, 12, 48, 12);//output
        }

        private void populatePoints() {
            inputPoints[0] = new Point(4, 6);
            inputPoints[1] = new Point(4, 18);
            outputPoints[0] = new Point(48, 12);
        }

    }

    public class NotGate : DigitalObject
    {

        public NotGate() {
            numInputs = 1;
            numOutputs = 1;

            inputs = new DigitalObject[numInputs];
            outputs = new DigitalObject[numOutputs];

            toolType = ToolboxStrip.DigiTool.NotGate;

            inputPoints = new Point[numInputs];
            outputPoints = new Point[numOutputs];

            populatePoints();
            populateLineDrawing();
        }

        private void populateLineDrawing() {
            lineDrawing = new LineDrawing[6];
            lineDrawing[0] = new LineDrawing(5, 0, 30, 12);//top line
            lineDrawing[1] = new LineDrawing(5, 0, 5, 24);//left
            lineDrawing[2] = new LineDrawing(5, 24, 30, 12);//bottom
            lineDrawing[3] = new LineDrawing(0, 12, 5, 12);//input 1
            lineDrawing[4] = new LineDrawing(36, 12, 41, 12);//output
            lineDrawing[5] = new LineDrawing(30, 10, 34, 14, 0, 360);//input 2
        }

        private void populatePoints() {
            inputPoints[0] = new Point(0, 12);
            outputPoints[0] = new Point(41, 12);
        }

    }

    public class Wire : DigitalObject
    {

        public Wire() {
            numInputs = 1;
            numOutputs = 1;

            inputs = new DigitalObject[numInputs];
            outputs = new DigitalObject[numOutputs];

            toolType = ToolboxStrip.DigiTool.Wire;

            inputPoints = new Point[numInputs];
            outputPoints = new Point[numOutputs];

            //populatePoints();
            //populateLineDrawing();
        }

        private void populateLineDrawing() {
            lineDrawing = new LineDrawing[7];
            lineDrawing[0] = new LineDrawing(9, 0, 15, 20, 270, 180);
            lineDrawing[1] = new LineDrawing(0, 0, 15, 0);
            lineDrawing[2] = new LineDrawing(0, 0, 0, 20);
            lineDrawing[3] = new LineDrawing(0, 20, 15, 20);
            lineDrawing[4] = new LineDrawing(0, 5, -5, 5);
            lineDrawing[4] = new LineDrawing(0, 15, -5, 15);
            lineDrawing[5] = new LineDrawing(24, 10, 30, 10);
        }

        private void populatePoints() {
            inputPoints[0] = new Point(5, 5);
            inputPoints[1] = new Point(5, 15);
            outputPoints[0] = new Point(20, 15);
        }

    }

}
