using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

namespace ToolboxStrip
{

    public enum DigiTool
    {
        Wire,
        Bus,
        AndGate,
        OrGate,
        NotGate,
        XorGate
    }

    public partial class ToolBoxStrip : UserControl
    {

        private bool isActiveControl = false;

        public ToolBoxStrip() {
            //ToolStripManager.Renderer = new ToolStripProfessionalRenderer(new TanColorTable());
            InitializeComponent();
            //toolStrip1.Renderer = new ToolStripProfessionalRenderer(new TanColorTable());
            //toolStrip1.Items.Clear();
            foreach (ToolStripButton tlb in toolStrip1.Items) {
                tlb.ImageAlign = ContentAlignment.MiddleLeft;
                tlb.ImageTransparentColor = Color.Magenta;
                //toolStrip1.Items.Add(tlb);
            }
        }


        private string panelTitle;

        [Category("Appearance"), Description("The title of the panel"), DefaultValue("ToolBox Title"), Browsable(true)]
        public string PanelTitle {
            get { return panelTitle; }
            set {
                panelTitle = value;
                lblTitle.Text = value;
            }
        }


        public DigiTool selectedTool;


        private void panel1_Paint(object sender, PaintEventArgs e) {
            if (isActiveControl) {
                lblTitle.ForeColor = Color.White;
                System.Drawing.Drawing2D.LinearGradientBrush bb = new System.Drawing.Drawing2D.LinearGradientBrush(
                    new Point(0, 0),
                    new Point(0, panel1.Height),
                    Color.FromArgb(59, 128, 237),
                    Color.FromArgb(49, 106, 197)
                );
                e.Graphics.FillRectangle(bb, 0, 0, panel1.Width, panel1.Height);
            }
            else {
                lblTitle.ForeColor = Color.Black;
                e.Graphics.FillRectangle(new SolidBrush(Color.FromArgb(204, 199, 186)),
                    0, 0, panel1.Width, panel1.Height);
            }
        }

        private void ToolBoxStrip_Paint(object sender, PaintEventArgs e) {
            e.Graphics.DrawLine(new Pen(Color.FromArgb(172, 168, 153)), 0, 0, this.Width, 0);
            e.Graphics.DrawLine(new Pen(Color.FromArgb(172, 168, 153)), this.Width - 1, 0,
                this.Width - 1, this.Height-1);
        }

        private void ToolBoxStrip_Enter(object sender, EventArgs e) {
            isActiveControl = true;
            //btnMinimise.FlatAppearance.MouseOverBackColor = Color.FromArgb(43, 91, 172);
            btnMinimise.FlatAppearance.MouseOverBackColor = Color.FromArgb(51, 157, 255);
            btnClose.FlatAppearance.MouseOverBackColor = Color.FromArgb(255, 100, 82);
            panel1.Refresh();
        }

        private void ToolBoxStrip_Leave(object sender, EventArgs e) {
            isActiveControl = false;
            btnMinimise.FlatAppearance.MouseOverBackColor = Color.FromArgb(156, 182, 231);
            btnClose.FlatAppearance.MouseOverBackColor = Color.FromArgb(234, 133, 109);
            panel1.Refresh();
        }

        private void toolStrip1_Click(object sender, EventArgs e) {
            this.Focus();
            panel1.Refresh();
        }

        private void btnMinimise_Paint(object sender, PaintEventArgs e) {
            if (isActiveControl) {
                e.Graphics.FillRectangle(Brushes.White, new Rectangle(3, 12, 8, 2));
            }
            else {
                e.Graphics.FillRectangle(Brushes.Black, new Rectangle(3, 12, 8, 2));
            }
        }

        private void btnClose_Paint(object sender, PaintEventArgs e) {
            int offset = 5;
            int offsetbar = 18 - 5;
            e.Graphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;
            if (isActiveControl) {
                e.Graphics.DrawLine(new Pen(Color.White, 2.0F), new Point(offset, offset), new Point(offsetbar, offsetbar));
                e.Graphics.DrawLine(new Pen(Color.White, 2.0F), new Point(offset, offsetbar), new Point(offsetbar, offset));
            }
            else {
                e.Graphics.DrawLine(new Pen(Color.Black, 2.0F), new Point(offset, offset), new Point(offsetbar, offsetbar));
                e.Graphics.DrawLine(new Pen(Color.Black, 2.0F), new Point(offset, offsetbar), new Point(offsetbar, offset));
            }
        }

        private void panel1_Click(object sender, EventArgs e) {
            this.Focus();
        }

        private void panel2_Click(object sender, EventArgs e) {
            this.Focus();
        }

        private void toolbox_button_click(object sender, EventArgs e) {
            foreach (ToolStripButton tlb in toolStrip1.Items) {
                if (tlb == sender) {
                    if (!tlb.Checked) tlb.Checked = true;

                    setSelectedTool(tlb);

                }
                else {
                    tlb.Checked = false;
                }
            }
        }

        private void setSelectedTool(ToolStripButton tlb) {
            switch (toolStrip1.Items.IndexOf(tlb)) {
                case 0:
                    selectedTool = DigiTool.Wire;
                    break;
                case 1:
                    selectedTool = DigiTool.Bus;
                    break;
                case 2:
                    selectedTool = DigiTool.AndGate;
                    break;
                case 3:
                    selectedTool = DigiTool.OrGate;
                    break;
                case 4:
                    selectedTool = DigiTool.NotGate;
                    break;
                case 5:
                    selectedTool = DigiTool.XorGate;
                    break;
                default:
                    break;
            }
        }



    }
}